#!/usr/bin/python
# -*- coding: utf-8 -*-

from .operators.imgdata import ImgData
from .operators import coco, csv, pascalvoc
import logging
import os

# set logger
logging.basicConfig()
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)


class Convertor:

    """ convert method implementation class """

    @staticmethod
    def coco2csv(dataset_dir: str,
                 coco_ann_dir: str,
                 save_dir: str):
        """ convert coco to csv

        :param dataset_dir: relative path current folder, or absolute path to the main folder of the image dataset
        :param coco_ann_dir: relative path current folder, or absolute path to the main folder of the annotated file
        :param save_dir: .csv file saving location
        :return: None
        """
        imgdataset = ImgData.extract(dataset_dir)
        coco_obj = coco.COCO(imgdataset.dataset)
        coco_obj.extract(coco_ann_dir)
        df = coco_obj.get_dataset()
        ann, clas = coco_obj.get_annotations()

        csv_obj = csv.CSV(df)
        csv_obj.set_annotations(ann)
        csv_obj.set_classes(clas)
        csv_fomatted = csv_obj.translate()
        csv_obj.archive(save_dir, csv_fomatted)

    @staticmethod
    def coco2voc(dataset_dir: str,
                 coco_ann_dir: str,
                 save_dir: str):
        """ convert coco to pascal VOC

        :param dataset_dir: relative path current folder, or absolute path to the main folder of the image dataset
        :param coco_ann_dir: relative path current folder, or absolute path to the main folder of the annotated file
        :param save_dir: .csv file saving location
        :return: None
        """
        if not os.path.exists(save_dir):
            os.makedirs(save_dir)

        imgdataset = ImgData.extract(dataset_dir)
        coco_obj = coco.COCO(imgdataset.dataset)
        coco_obj.extract(coco_ann_dir)
        df = coco_obj.get_dataset()
        ann, cls = coco_obj.get_annotations()

        voc_obj = pascalvoc.PascalVOC(df)
        voc_obj.set_annotations(ann)
        voc_obj.set_classes(cls)
        for xml, name in voc_obj.translate():
            file_dir = save_dir + '/' + name.split('.')[0]+'.xml'
            voc_obj.archive(file_dir, xml)

    @staticmethod
    def csv2coco(dataset_dir: str,
                 csv_ann_dir: str,
                 save_dir: str):
        """ convert .csv into coco

        :param dataset_dir: relative path current folder, or absolute path to the main folder of the image dataset
        :param csv_ann_dir: relative path current folder, or absolute path to the main folder of the annotated file
        :param save_dir: .csv file saving location
        :return: None
        """
        imagedataset = ImgData.extract(dataset_dir)
        csv_obj = csv.CSV(imagedataset.dataset)
        csv_obj.extract(csv_ann_dir)
        df = csv_obj.get_dataset()
        ann, cls = csv_obj.get_annotations()

        coco_obj = coco.COCO(df)
        coco_obj.set_annotations(ann)
        coco_obj.set_classes(cls)
        data = coco_obj.translate()
        coco_obj.archive(save_dir, data)

    @staticmethod
    def csv2voc(dataset_dir: str,
                csv_ann_dir: str,
                save_dir: str):
        """ convert .csv into pascal VOC

        :param dataset_dir: relative path current folder, or absolute path to the main folder of the image dataset
        :param csv_ann_dir: relative path current folder, or absolute path to the main folder of the annotated file
        :param save_dir: .csv file saving location
        :return: None
        """
        if not os.path.exists(save_dir):
            os.makedirs(save_dir)

        imagedataset = ImgData.extract(dataset_dir)
        csv_obj = csv.CSV(imagedataset.dataset)
        csv_obj.extract(csv_ann_dir)
        df = csv_obj.get_dataset()
        ann, cls = csv_obj.get_annotations()

        voc_obj = pascalvoc.PascalVOC(df)
        voc_obj.set_annotations(ann)
        voc_obj.set_classes(cls)
        for xml, name in voc_obj.translate():
            file_dir = save_dir + '/' + name.split('.')[0]+'.xml'
            voc_obj.archive(file_dir, xml)

    @staticmethod
    def voc2coco(dataset_dir: str,
                 voc_ann_dir: str,
                 save_dir: str):
        """ convert pascal VOC into coco

        :param dataset_dir: relative path current folder, or absolute path to the main folder of the image dataset
        :param voc_ann_dir: relative path current folder, or absolute path to the main folder of the annotated file
        :param save_dir: .csv file saving location
        :return: None
        """
        imagedataset = ImgData.extract(dataset_dir)
        voc_obj = pascalvoc.PascalVOC(imagedataset.dataset)
        voc_obj.extract(voc_ann_dir)
        df = voc_obj.get_dataset()
        ann, cls = voc_obj.get_annotations()

        coco_obj = coco.COCO(df)
        coco_obj.set_annotations(ann)
        coco_obj.set_classes(cls)
        data = coco_obj.translate()
        coco_obj.archive(save_dir, data)

    @staticmethod
    def voc2csv(dataset_dir: str,
                voc_ann_dir: str,
                save_dir: str):
        """ convert pascal VOC into .csv

        :param dataset_dir: relative path current folder, or absolute path to the main folder of the image dataset
        :param voc_ann_dir: relative path current folder, or absolute path to the main folder of the annotated file
        :param save_dir: .csv file saving location
        :return: None
        """
        imagedataset = ImgData.extract(dataset_dir)
        voc_obj = pascalvoc.PascalVOC(imagedataset.dataset)
        voc_obj.extract(voc_ann_dir)
        df = voc_obj.get_dataset()
        ann, cls = voc_obj.get_annotations()

        csv_obj = csv.CSV(df)
        csv_obj.set_annotations(ann)
        csv_obj.set_classes(cls)
        csv_fomatted = csv_obj.translate()
        csv_obj.archive(save_dir, csv_fomatted)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import logging
import sys

from .operators.imgdata import ImgData
from .operators import coco, csv, pascalvoc

# setup logger
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

"""
### obj_lis : attributes ###
{
    ["classes" : [int, ..],
    "bbox" : [[(x_min, y_min), (x_max, y_max)], ..],
    "image_id" : int,
    "path: : str], ..
}
"""


class Sample:
    """ Data Annotation file sampling interface """

    @staticmethod
    def show_samples(data_path: str,
                     ann_path: str,
                     num_of_samples: int = 5,
                     ann_type: str = 'coco'):
        """ render set of random images from dataset.

        :param data_path: relative path current folder, or absolute path to the main folder of the image dataset
        :param ann_path: relative path current folder, or absolute path to the main folder of the annotated file
        :param ann_type: one of type from ['coco', 'voc', 'csv', 'yolo']
        :param num_of_samples: number of sample images in integer format.
        :return: render sequence of images.
        """

        global obj
        imgdataset = ImgData.extract(data_path)
        if ann_type == 'coco':
            obj = coco.COCO(imgdataset.dataset)
        elif ann_type == 'voc':
            obj = pascalvoc.PascalVOC(imgdataset.dataset)
        elif ann_type == 'csv':
            obj = csv.CSV(imgdataset.dataset)
        elif ann_type == 'yolo':
            obj = csv.IOperator(imgdataset.dataset)
        else:
            logger.error(f"\nERROR: '{ann_type}' is not a valid annotation type.")
            sys.exit(1)

        obj.extract(ann_path)
        obj_list = obj.sample(num_of_samples)
        cat_dict = obj.classes
        for img_obj in obj_list:
            path = img_obj["path"]
            obj_data = img_obj["bbox"]
            cat_name = [cat_dict[j] for j in img_obj["classes"]]
            obj.render(path, obj_data, cat_name)
        return

    @staticmethod
    def describe_data(data_path: str):
        """
        give a summary of a image dataset
        :param data_path: absolute or relative path to image dataset main folder
        :return: log of summary
        """
        img_dataset = ImgData.extract(data_path)
        data_dict = img_dataset.describe()
        log_st = Sample.descFormat("image data summary", data_dict)
        logger.info("\n"+log_st)
        return

    @staticmethod
    def describe_ann(data_path: str,
                     ann_path: str, ann_type: str = 'coco'):
        """
        give summary of annotated dataset
        :param data_path: absolute or relative path to image dataset main folder
        :param ann_path: absolute or relative path to image annotation file or folder
        :param ann_type: annotation format [coco, voc, csv, yolo]
        :return: log of summary
        """
        imgdataset = ImgData.extract(data_path)
        if ann_type == 'coco':
            obj = coco.COCO(imgdataset.dataset)
        elif ann_type == 'voc':
            obj = pascalvoc.PascalVOC(imgdataset.dataset)
        elif ann_type == 'csv':
            obj = csv.CSV(imgdataset.dataset)
        elif ann_type == 'yolo':
            obj = csv.IOperator(imgdataset.dataset)
        else:
            logger.error(f"\n ERROR : {ann_type} is not a valid annotation type.")
            sys.exit(1)

        obj.extract(ann_path)
        log_data = obj.describe()
        log_st = Sample.descFormat("image annotation summary", log_data)
        logger.info("\n"+log_st)
        return

    @staticmethod
    def descFormat(header, data):
        """
        give formatted string for data

        :param header: header name of the summary
        :param data: dictionary of data {field : value}
        :return: formatted string of all data.
        """
        log_string = ""
        log_header = "{0:^80s}\n{1:s}\n".format(header.upper(), '=' * 80)
        log_string += log_header

        wd = Sample.__getMax(data.keys())
        log_data = ""
        for sec in data:
            if type(data[sec]) == str:
                st = "{0:<{1:d}s} : {2:}\n".format(sec, wd, data[sec])
                log_data += st
            elif type(data[sec]) == int:
                st = "{0:<{1:d}s} : {2:d}\n".format(sec, wd, data[sec])
                log_data += st
            elif type(data[sec]) == dict:
                sub_wd = Sample.__getMax(data[sec].keys())
                sub_topic = "{0:<{1:d}s} :\n".format(sec, wd)
                log_data += sub_topic
                for sub_elem in data[sec]:
                    sub_data = "{0:<{1:d}s} > {2:<{3:d}} : {4:}\n".format(" ", wd, sub_elem, sub_wd, data[sec][sub_elem])
                    log_data += sub_data
        log_data += "{0:s}\n".format("="*80)
        log_string += log_data
        return log_string

    @staticmethod
    def __getMax(ls):
        """

        :param ls: list of string
        :return: length of maximum length string in the list
        """
        elem = max(ls, key=len)
        return len(elem)




#!/usr/bin/python
# -*- coding: utf-8 -*-

from abc import ABCMeta, abstractmethod
import matplotlib.pyplot as plt
import cv2
import pandas as pd
import logging
import random
import sys

# set logger
logging.basicConfig()
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

""":param
ann_df attributes:
    - obj_id : int
    - image_id : int
    - class_id : int
    - x_min : int
    - y_min : int
    - x_max : int
    - y_max : int
"""
""":param
render format
    - image path
    - box : [[(x_min, y_min), (x_max, y_max)], ...]
    - classes : [str, ...]
"""

""":param
classes
{class_id[int] : class_name[str], ... }
"""


class IOperator(object):
    """ Operator Abstract class """

    def __init__(self, dataset):
        self._dataset = dataset

    def set_dataset(self, df):
        """
        :param df: pandas.DataFrame type object with attr. defined in the ImgData.py file

        save new dataset into the objects' private variable.
        """
        if type(df) is pd.DataFrame:
            self._dataset = df
        else:
            logger.error(f"\n ERROR : Data type of df : {type(df)} not compatible with database object.")

    def get_dataset(self):
        """
        :return pandas.DataFrame
        """
        return self._dataset

    def get_annotations(self):
        """
        :return: data in annotation file [pd.Dataframe] & classes in a dictionary
        """
        try:
            if self.annotations.shape[0] and self.classes:
                return self.annotations, self.classes
            else:
                # TODO : check here.
                return
        except Exception as error:
            logger.exception(error)
            sys.exit(1)

    def set_annotations(self, ann):
        """

        :param ann: data in annotation file [pd.Dataframe]
        :return:
        """
        self.annotations = ann

    def set_classes(self, classes):
        """

        :param classes: classes in a dictionary
        :return:
        """
        self.classes = classes

    @abstractmethod
    def extract(self, path: str):
        raise NotImplementedError

    @abstractmethod
    def translator(self):
        raise NotImplementedError

    @abstractmethod
    def archive(self, location, data):
        raise NotImplementedError



    def describe(self):
        """
        give of summary of data folders
        :return: (dictionary)
        desc_dict = { number of images : str,
                    number of folders : int,
                    folder image count : dict{}
                    }
        """
        desc_dict = {}

        num_of_imgs = self._dataset.shape[0]
        desc_dict["number of images"] = num_of_imgs

        fld_img_cnt_df = self._dataset.loc[:, ["name", "folder"]].groupby("folder").count()
        desc_dict["folder image counts"] = fld_img_cnt_df.to_dict()["name"]

        num_sizes = self._dataset.loc[:, ["width", "height"]].nunique().tolist()
        if num_sizes == [1, 1]:
            img_size = self._dataset.loc[1, ["width", "height"]].tolist()
            desc_dict["number of image sizes"] = 1
            desc_dict["image_size"] = "{:0} X {:1}".format(img_size[0], img_size[1])
        else:
            desc_dict["number of image sizes"] = len(self._dataset.groupby(["width", "height"]))

        desc_dict["number of object classes"] = len(self.classes)
        desc_dict["object classes"] = " | ".join(self.classes.values())

        desc_dict["number of objects"] = self.annotations.shape[0]

        class_gb = self.annotations.loc[:, ["obj_id", "class_id"]].groupby("class_id").count().reset_index()
        class_gb["classes"] = class_gb["class_id"].map(self.classes)
        class_cnt = class_gb.set_index("classes")["obj_id"].to_dict()
        desc_dict["class object count"] = class_cnt
        return desc_dict

    def sample(self, numOfSamples):
        """
        choose set of images randomly and get bounding boxes of them
        :return: dictionary list of [{"image_id" : "name", "classes" : [], "categories" : []}]
        """
        numOfrecords, _ = self._dataset.shape
        rnd_numbers = sorted(random.sample(range(0, numOfrecords), numOfSamples))
        sample_df = self._dataset.iloc[rnd_numbers, :]
        image_list = list(sample_df.loc[:, "image_id"].values)
        image_paths = list(sample_df.loc[:, "path"].values)
        sampled_anns = self.annotations.loc[self.annotations.loc[:, "image_id"].isin(image_list), :]
        final_list = []
        for image_id, image_path in zip(image_list, image_paths):
            ann_for_image = sampled_anns.loc[sampled_anns.loc[:, "image_id"] == image_id, :]
            spares_list = ann_for_image.values.tolist()
            ordered_dict = self.__listGen(spares_list)
            ordered_dict["image_id"] = image_id
            ordered_dict["path"] = image_path
            final_list.append(ordered_dict)
        return final_list

    def render(self, path: str, boxes: list, cls: list, rect_th=1, text_size=0.5, text_th=1):
        """ show annotated image

        :param path: directory to image
        :param boxes: all the bounding boxes in [[(),()], ...]
        :param cls: list of class names
        :param rect_th: thickness of he box :int
        :param text_size: font size
        :param text_th: thickness of the text :int
        :return: matplotlib.pyplot.plt object / a image.
        """
        img = cv2.imread(path)
        img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)

        for i in range(len(boxes)):
            # print(boxes[i][0], boxes[i][1])
            cv2.rectangle(img, boxes[i][0], boxes[i][1], color=(14, 14, 14), thickness=rect_th)
            cv2.putText(img, cls[i],
                        boxes[i][0], cv2.FONT_HERSHEY_COMPLEX,
                        text_size, color=(1, 1, 1), thickness=text_th)
        plt.figure(figsize=(30, 30))
        plt.imshow(img)
        plt.xticks([])
        plt.yticks([])
        plt.show()
        return

    def __listGen(self, data_list):
        """

        :param data_list: [obj_id, class_id, class_id , x_min, y_min, x_max, y_max]
        :return: two list {"classes" : [classes, ..] , "bbox" : [[(x_min, y_min), (x_max, y_max)], ..]}
        """
        bounding_boxes = []
        classes = []
        for obj in data_list:
            classes.append(obj[2])
            bounding_boxes.append([(obj[3], obj[4]), (obj[5], obj[6])])
        final_dict = {"classes": classes, "bbox": bounding_boxes}
        return final_dict


